close all;

Prefix = '2015.06.05 16-53-22 Walk Test';

PathPrefix = strcat('..\',Prefix);
DataPath = strcat(PathPrefix,'.Log.xlsx');
MoviePath = strcat(PathPrefix, '.Truth.mp4');
OutMoviePath = strcat(PathPrefix, '.MovPlot.avi');

if ~exist('MovFrames','var')
    MovObj = VideoReader(MoviePath);
    MovRate = MovObj.FrameRate;
    MovFrames = struct('cdata',zeros(MovObj.Height,MovObj.Width,3,'uint8'),...
        'colormap',[]);
    Frame = 1;
    while hasFrame(MovObj);
        MovFrames(Frame).cdata = readFrame(MovObj);
        Frame = Frame + 1;
    end
end

if ~exist('LogData','var')
    LogData = xlsread(DataPath);
end

SampleNoCol = 3;
DispCol = 7;
ConfCol = 8;

DataInSeconds = LogData(mod(LogData(:,SampleNoCol),250)==1,:); % 250 samples/sec
BeginSecond = DataInSeconds(1,SampleNoCol);
Index = (DataInSeconds(:,SampleNoCol) - BeginSecond)/250;
Index = Index + 1;
DataInSeconds(:,SampleNoCol) = Index;

TextFontSize = 7.5;

f = figure;
figtitle(Prefix);
set(f,'position',[10 10 1000 1000]);
OutFramesIndex = 1;

writerObj = VideoWriter(OutMoviePath);
writerObj.FrameRate = MovRate;
open(writerObj);

SubPlotR = 14;
SubPlotC = 12;
PlotRows = 3;

for Frame = 1:length(MovFrames)
    clf;
    subplot(SubPlotR,SubPlotC,1:SubPlotR*SubPlotC-3*SubPlotC);
    imshow(MovFrames(Frame).cdata);
    title(Prefix);
        
    subplot(SubPlotR,SubPlotC,(SubPlotR*SubPlotC-3*SubPlotC)+1:SubPlotR*SubPlotC);
    Second = floor(Frame / MovRate);
    if Second > 0
        Beg = max(0, Second - 5);
        End = min(max(Index), Second + 5);
        D2 = DataInSeconds(DataInSeconds(:,SampleNoCol) >= Beg,:);
        D2 = D2(D2(:,SampleNoCol) <= End,:);
        Idx2 = min(D2(:,SampleNoCol)):max(D2(:,SampleNoCol));
        plot(Idx2,D2(:,DispCol),'--k',Idx2,D2(:,ConfCol),'-.r');
        legend('Displacement','Confirmation');
        ylim([-.5,1.5]);
        line([Second Second],ylim);
    end
    
    OutFrame = getframe(gcf);
    writeVideo(writerObj, OutFrame);
end

% for Second = min(Index):max(Index)
%     clf;
%     
%     BegFrame = floor((Second - 1) * MovRate) + 1;
%     for Frame = BegFrame : floor(BegFrame + MovRate);
%         SubPlotR = 14;
%         SubPlotC = 12;
%         PlotRows = 3;
%         
%         subplot(SubPlotR,SubPlotC,1:SubPlotR*SubPlotC-3*SubPlotC);
%         %subplot(2,1,1);
%         imshow(MovFrames(Frame).cdata);
%         title(Prefix);
%         
%         subplot(SubPlotR,SubPlotC,(SubPlotR*SubPlotC-3*SubPlotC)+1:SubPlotR*SubPlotC);
%         %subplot(2,1,2);
%         Beg = max(0, Second - 5);
%         End = min(max(Index), Second + 5);
%         D2 = DataInSeconds(DataInSeconds(:,SampleNoCol) >= Beg,:);
%         D2 = D2(D2(:,SampleNoCol) <= End,:);
%         Idx2 = min(D2(:,SampleNoCol)):max(D2(:,SampleNoCol));
%         plot(Idx2,D2(:,DispCol),'--k',Idx2,D2(:,ConfCol),'-.r');
%         %title(Prefix);
%         legend('Displacement','Confirmation');
%         ylim([-.5,1.5]);
%         line([Second Second],ylim);
% 
%         OutFrame = getframe(gcf);
%         writeVideo(writerObj, OutFrame);
%         
%     end
% end

close(writerObj);

