close all;

Prefix = '2015.07.22 12-15-25 Walk Test - Driveway';
PathPrefix = strcat('..\',Prefix);

if ~exist('LogData','var')
    [LogData,~,AllData] = xlsread(strcat(PathPrefix, '.Log.xlsx'));
end

[~, NCols] = size(AllData);
TagCol = -1;
for I = 1:NCols
    if strcmp(AllData(1,I),'#c')
        TagCol = I;
        break;
    end
end

if TagCol < 1
    fprintf('Tag column not found\nTerminating\n');
    return;
end

SampleNoCol = TagCol + 1;
DispCol = TagCol + 5;
ConfCol = TagCol + 6;

DataInSeconds = LogData(mod(LogData(:,SampleNoCol),250)==1,:); % 250 samples/sec
BeginSecond = DataInSeconds(1,SampleNoCol);
Index = (DataInSeconds(:,SampleNoCol) - BeginSecond)/250;
Index = Index + 1;
DataInSeconds(:,SampleNoCol) = Index;

TextFontSize = 7.5;

% figure;
% for I = 1:max(OutFrames);
%     clf;
%     imshow(OutFrames(I).cdata);
% end;

TickSpacing = 10; % seconds

figure('units','normalized','outerposition',[0 0 1 1]);
figtitle(Prefix);
subplot(2,1,1);
plot(Index, DataInSeconds(:,DispCol),'g');
title(Prefix);
ylim([-.5,1.5]);
legend('Displacement');
xlabel('Time (seconds)');
ax = gca;
ax.XTick = 0:TickSpacing:max(Index);

for Second = 2: max(Index)
    if (DataInSeconds(Second, DispCol) ~= DataInSeconds(Second-1, DispCol))
        h = text(Index(Second), DataInSeconds(Second, DispCol), sprintf('%02d:%02d',floor(Index(Second)/60), mod(Index(Second),60)) ...
            ,'HorizontalAlignment','right');
        h.FontSize = TextFontSize;
    end
end

subplot(2,1,2);
plot(Index, DataInSeconds(:,ConfCol),'r');
ylim([-.5,1.5]);
legend('Confirmation');
xlabel('Time (seconds)');
ax = gca;
ax.XTick = 0:TickSpacing:max(Index);

for Second = 2: max(Index)
    if (DataInSeconds(Second, ConfCol) ~= DataInSeconds(Second-1, ConfCol))
        h = text(Index(Second), DataInSeconds(Second, ConfCol), sprintf('%02d:%02d',floor(Index(Second)/60), mod(Index(Second),60)), ...
            'HorizontalAlignment','left');
        h.FontSize = TextFontSize;
    end
end

FigType = '.DispConf';

savefig(strcat(PathPrefix,FigType,'.fig'));
saveas(gcf,strcat(PathPrefix,FigType,'.png'));

% figure('units','normalized','outerposition',[0 0 1 1]);
% plot(Index, DataInSeconds(:,7),'g',Index, DataInSeconds(:,8),'r');
% ylim([-.5,1.5]);
% legend('Displacement','Confirmation');
% xlabel('Time (seconds)');
% ax = gca;
% ax.XTick = 0:TickSpacing:max(Index);
% 
% for I = 2: max(Index)
%     if (DataInSeconds(I,7) ~= DataInSeconds(I-1,7))
%         text(Index(I), DataInSeconds(I,7), sprintf('%i',Index(I)),'HorizontalAlignment','right');
%     end
% end
% for I = 2: max(Index)
%     if (DataInSeconds(I,8) ~= DataInSeconds(I-1,8))
%         text(Index(I), DataInSeconds(I,8), sprintf('%i',Index(I)),'HorizontalAlignment','left');
%     end
% end
% 
% savefig(strcat(PathPrefix,'Fig2.fig'));
% saveas(gcf,strcat(PathPrefix,'Fig2.png'));
